/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.jetty;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.DefaultHandler;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.security.SslSocketConnector;
import org.mortbay.jetty.webapp.WebAppContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * JettyHelper provides convenience methods for starting Jetty instances with
 * compact configuration options
 * 
 * @author Timo Westkamper
 * @author Marko Lavikainen
 * @version $Id$
 */
public final class JettyHelper {

    private JettyHelper() {
    }

    private static final Logger logger = LoggerFactory
            .getLogger(JettyHelper.class);

    private static final Map<Integer, Server> servers = new HashMap<Integer, Server>();

    private static final File keystoreLocation;

    static {
        try {
            keystoreLocation = File.createTempFile("keystore", null);
            InputStream in = JettyHelper.class.getResourceAsStream("/keystore");
            OutputStream out = new FileOutputStream(keystoreLocation);
            try {
                IOUtils.copy(in, out);
            } finally {
                out.close();
                in.close();
            }
        } catch (IOException e) {
            throw new JettyException(e.getMessage(), e);
        }

    }

    public static boolean runningAt(int port) {
        return servers.containsKey(port);
    }

    public static JettyConfig startJetty(JettyConfig config) {
        WebAppContext webappcontext = new WebAppContext();
        webappcontext.setContextPath(config.contextPath);
        webappcontext.setWar(config.warPath);
        if (new File(config.warPath).isFile()) {
            webappcontext.setExtractWAR(true);
        }
        return startJetty(config, webappcontext);
    }

    public static JettyConfig startJetty(String warPath, String contextPath,
            int port, int securePort) {
        return startJetty(new JettyConfig(warPath, contextPath, port,
                securePort));
    }

    public static JettyConfig startJetty(int port, int securePort,
            WebAppContext... contexts) {
        return startJetty(new JettyConfig(port, securePort), contexts);
    }

    public static JettyConfig startJetty(JettyConfig config,
            WebAppContext... contexts) { // NOPMD

        try {

            if (servers.containsKey(config.port)) {
                logger.error("Already one Jetty instance running on port "
                        + config.port, new JettyException());
                return null;
            }

            if (contexts == null || contexts.length == 0) {
                throw new IllegalArgumentException();
            }

            final Server server = new Server();
            servers.put(config.port, server);

            List<Connector> connectors = new ArrayList<Connector>(2);

            // http
            if (config.port > 0) {
                SelectChannelConnector httpConnector = new SelectChannelConnector();
                httpConnector.setPort(config.port);
                httpConnector.setConfidentialPort(config.securePort);
                connectors.add(httpConnector);
            }

            // https
            if (config.securePort > 0) {
                SslSocketConnector httpsConnector = new SslSocketConnector();
                httpsConnector.setPort(config.securePort);
                httpsConnector.setMaxIdleTime(60000);
                httpsConnector.setKeystore(keystoreLocation.getAbsolutePath());
                httpsConnector
                        .setPassword("OBF:1vny1zlo1x8e1vnw1vn61x8g1zlu1vn4");
                httpsConnector.setKeyPassword("OBF:1u2u1wml1z7s1z7a1wnl1u2g");
                connectors.add(httpsConnector);
            }

            server.setConnectors(connectors.toArray(new Connector[connectors
                    .size()]));

            List<Handler> handlers = new LinkedList<Handler>();
            for (WebAppContext context : contexts) {
                handlers.add(context);
            }
            handlers.add(new DefaultHandler());
            server.setHandlers(handlers.toArray(new Handler[handlers.size()]));

            server.start();
            logger.info("Jetty running on port " + config.port);
            return config.setServer(server);
        } catch (Exception e) {
            throw new JettyException(e);
        }
    }

    public static void stopJettyAtPort(int port) { // NOPMD
        Server server = servers.remove(port);
        if (server != null) {

            try {
                server.stop();
            } catch (Exception e) {
                throw new JettyException("Unable to stop server", e);
            }
            logger.info("Stopped Jetty on port " + port);
        } else {
            logger.warn("No Jetty instance running on port " + port,
                    new JettyException());
        }
    }

}
